@@ -750,6 +750,14 @@ def test_rasterize_invalid_value(basic_geometry):
                 gdal_version.at_least("3.5"), reason="GDAL regression? Works with 3.4.3"
             ),
         ),
+        pytest.param(
+            "int8",
+            -128,
+            marks=pytest.mark.skipif(
+                not gdal_version.at_least("3.7"),
+                reason="int8 support added in GDAL 3.7",
+            ),
+        ),
         ("uint8", 255),
         ("uint16", 65535),
         ("float32", 1.434532),
@@ -781,11 +789,12 @@ def test_rasterize_supported_dtype(dtype, default_value, basic_geometry):
 def test_rasterize_unsupported_dtype(basic_geometry):
     """Unsupported types should all raise exceptions."""
     unsupported_types = (
-        ('int8', -127),
-        ('float16', -9343.232)
+        ('float16', -9343.232),
     )
     if not gdal_version.at_least("3.5"):
         unsupported_types += (('int64', 20439845334323),)
+    if not gdal_version.at_least("3.7"):
+        unsupported_types += (('int8', -127),)
 
     for dtype, default_value in unsupported_types:
         with pytest.raises(ValueError):
@@ -1263,3 +1272,31 @@ def test_zz_no_dataset_leaks(capfd):
         env._dump_open_datasets()
         captured = capfd.readouterr()
         assert not captured.err
+
+
+def test_sieve_bands(pixelated_image, pixelated_image_file):
+    """Verify fix for gh-2782."""
+    truth = sieve(pixelated_image, 9)
+
+    with rasterio.open(pixelated_image_file) as src:
+        assert np.array_equal(truth, sieve(rasterio.band(src, [1]), 9))
+
+        # Mask band should also work but will be a no-op
+        assert np.array_equal(
+            pixelated_image,
+            sieve(rasterio.band(src, [1]), 9, mask=rasterio.band(src, 1))
+        )
+
+
+def test_sieve_dataset(pixelated_image, pixelated_image_file):
+    """Verify fix for gh-2782."""
+    truth = sieve(pixelated_image, 9)
+
+    with rasterio.open(pixelated_image_file) as src:
+        assert np.array_equal(truth, sieve(src, 9))
+
+        # Mask band should also work but will be a no-op
+        assert np.array_equal(
+            pixelated_image,
+            sieve(src, 9, mask=rasterio.band(src, 1))
+        )
